#include <u.h>
#include <libc.h>
#include <ureg.h>
#include <bio.h>
#include <ndb.h>

#include "dat.h"
#include "fns.h"
#include "linux.h"

typedef struct Etcfile Etcfile;
struct Etcfile
{
	Ufile;
	int	q;
	vlong	lastoff;
	char	*data;
};

enum
{
	Qpasswd,
	Qshadow,
	Qgroup,
	Qresolv,
};

static struct {
	int mode;
	char *name;
	int rdev;
	char *link;
} etcdevtab[] = {
	[Qpasswd]	0666|S_IFREG,	"passwd",
	[Qshadow]	0600|S_IFREG,	"shadow",
	[Qgroup]	0666|S_IFREG,	"group",
	[Qresolv]	0666|S_IFREG,	"resolv.conf",
};


static char *
genpasswd(void)
{
	Fmt f;

	char *user, *home;
	static char *std = 
		"root:x:0:0:root:/root:/bin/bash\n"
		"daemon:x:1:1:daemon:/usr/sbin:/bin/sh\n"
		"bin:x:2:2:bin:/bin:/bin/sh\n"
		"sys:x:3:3:sys:/dev:/bin/sh\n"
		"sync:x:4:65534:sync:/bin:/bin/sync\n"
		"games:x:5:60:games:/usr/games:/bin/sh\n"
		"man:x:6:12:man:/var/cache/man:/bin/sh\n"
		"lp:x:7:7:lp:/var/spool/lpd:/bin/sh\n"
		"mail:x:8:8:mail:/var/mail:/bin/sh\n"
		"news:x:9:9:news:/var/spool/news:/bin/sh\n"
		"uucp:x:10:10:uucp:/var/spool/uucp:/bin/sh\n"
		"proxy:x:13:13:proxy:/bin:/bin/sh\n"
		"www-data:x:33:33:www-data:/var/www:/bin/sh\n"
		"backup:x:34:34:backup:/var/backups:/bin/sh\n"
		"list:x:38:38:Mailing List Manager:/var/list:/bin/sh\n"
		"irc:x:39:39:ircd:/var/run/ircd:/bin/sh\n"
		"gnats:x:41:41:Gnats Bug-Reporting System (admin):/var/lib/gnats:/bin/sh\n"
		"nobody:x:65534:65534:nobody:/nonexistent:/bin/sh\n"
		"libuuid:x:100:101::/var/lib/libuuid:/bin/sh\n"
		"Debian-exim:x:101:103::/var/spool/exim4:/bin/false\n"
		"messagebus:x:103:110::/var/spool/exim4:/bin/false\n"
		"statd:x:102:65534::/var/lib/nfs:/bin/false\n";

	fmtstrinit(&f);
	fmtprint(&f, "%s", std);
	if((user = getuser()) == nil)
		sysfatal("getuser() failed");
	if((home = getenv("home")) == nil)
		sysfatal("$home not set");
	fmtprint(&f, "%s:x:1000:1000:%s:/n/9/%s:/bin/bash\n", user, user, home);
	free(home);
	return fmtstrflush(&f);
}

static char *
genshadow(void)
{
	Fmt f;

	char *user;
	static char *std = 
		"root:*:15205:0:99999:7:::\n"
		"daemon:*:15205:0:99999:7:::\n"
		"bin:*:15205:0:99999:7:::\n"
		"sys:*:15205:0:99999:7:::\n"
		"sync:*:15205:0:99999:7:::\n"
		"games:*:15205:0:99999:7:::\n"
		"man:*:15205:0:99999:7:::\n"
		"lp:*:15205:0:99999:7:::\n"
		"mail:*:15205:0:99999:7:::\n"
		"news:*:15205:0:99999:7:::\n"
		"uucp:*:15205:0:99999:7:::\n"
		"proxy:*:15205:0:99999:7:::\n"
		"www-data:*:15205:0:99999:7:::\n"
		"backup:*:15205:0:99999:7:::\n"
		"list:*:15205:0:99999:7:::\n"
		"irc:*:15205:0:99999:7:::\n"
		"gnats:*:15205:0:99999:7:::\n"
		"nobody:*:15205:0:99999:7:::\n"
		"libuuid:!:15205:0:99999:7:::\n"
		"Debian-exim:!:15205:0:99999:7:::\n"
		"messagebus:!:15205:0:99999:7:::\n"
		"statd:*:15205:0:99999:7:::\n";

	fmtstrinit(&f);
	fmtprint(&f, "%s", std);
	if((user = getuser()) == nil)
		sysfatal("getuser() failed");
	fmtprint(&f, "%s:*:15205:0:99999:7:::\n", user);
	return fmtstrflush(&f);
}

static char *
gengroup(void)
{
	Fmt f;

	char *user;
	static char *std = 
		"root:x:0:\n"
		"daemon:x:1:\n"
		"bin:x:2:\n"
		"sys:x:3:\n"
		"adm:x:4:\n"
		"tty:x:5:\n"
		"disk:x:6:\n"
		"lp:x:7:\n"
		"mail:x:8:\n"
		"news:x:9:\n"
		"uucp:x:10:\n"
		"man:x:12:\n"
		"proxy:x:13:\n"
		"kmem:x:15:\n"
		"dialout:x:20:\n"
		"fax:x:21:\n"
		"voice:x:22:\n"
		"cdrom:x:24:steve\n"
		"floppy:x:25:steve\n"
		"tape:x:26:\n"
		"sudo:x:27:\n"
		"audio:x:29:steve\n"
		"dip:x:30:steve\n"
		"www-data:x:33:\n"
		"backup:x:34:\n"
		"operator:x:37:\n"
		"list:x:38:\n"
		"irc:x:39:\n"
		"src:x:40:\n"
		"gnats:x:41:\n"
		"shadow:x:42:\n"
		"utmp:x:43:\n"
		"video:x:44:steve\n"
		"sasl:x:45:\n"
		"plugdev:x:46:steve\n"
		"staff:x:50:\n"
		"games:x:60:\n"
		"users:x:100:\n"
		"nogroup:x:65534:\n"
		"libuuid:x:101:\n"
		"crontab:x:102:\n"
		"Debian-exim:x:103:\n"
		"mlocate:x:104:\n"
		"ssh:x:105:\n"
		"messagebus:x:106:\n"
		"utempter:x:107:\n"
		"lpadmin:x:108:\n"
		"fuse:x:109:\n"
		"messagebus:x:110:\n";

	fmtstrinit(&f);
	fmtprint(&f, "%s", std);
	if((user = getuser()) == nil)
		sysfatal("getuser() failed");
	fmtprint(&f, "%s:x:1000:\n", user);
	return fmtstrflush(&f);
}

static char *
genresolv(void)
{
	Fmt f;
	Ndb *db;
	Ndbtuple *t, *tt;
	char *sysname;
	static char *query[] = { "dnsdomain", "dns" };

	fmtstrinit(&f);

	if((sysname = getenv("sysname")) == nil)
		sysfatal("$sysname not set");
	if((db = ndbopen(nil)) == nil)
		sysfatal("can't open ndb: %r");
	tt = ndbipinfo(db, "sys", sysname, query, nelem(query));
	for(t = tt; t; t = t->entry){
		if(strcmp(t->attr, "dns") == 0)
			fmtprint(&f, "nameserver %s\n", t->val);
		if(strcmp(t->attr, "dnsdomain") == 0)
			fmtprint(&f, "domain %s\n", t->val);
	}
	free(sysname);
	return fmtstrflush(&f);
}

static int
path2q(char *path)
{
	int q;
	char *p;

	if((p = strrchr(path, '/')) == nil)
		return -1;
	for(q = 0; q < nelem(etcdevtab); q++)
		if(strcmp(p+1, etcdevtab[q].name) == 0)
			return q;
	return -1;
}

static int
openetc(char *path, int omode, int, Ufile **pf)
{
	Etcfile *mf;
	int q;

	if((q = path2q(path)) < 0)
		return -ENOENT;
	if((etcdevtab[q].mode & S_IFLNK) == S_IFLNK)
		return -SYMLINK;

	mf = kmallocz(sizeof(*mf), 1);
	mf->ref = 1;
	mf->omode = omode;
	mf->path = kstrdup(path);
	mf->fd = -1;
	mf->dev = ETCDEV;
	mf->q = q;

	switch(q){
	case Qpasswd:
		mf->data = genpasswd();
		break;
	case Qshadow:
		mf->data = genshadow();
		break;
	case Qgroup:
		mf->data = gengroup();
		break;
	case Qresolv:
		mf->data = genresolv();
		break;
	default:
		abort();	/* cannot happen */
		break;
	}

	*pf = mf;
	return 0;
}

static int
closeetc(Ufile *file)
{
	Etcfile *ef = (Etcfile *)file;

	free(ef->data);
	return 0;
}

static int
readstr(ulong off, char *buf, ulong n, char *str)
{
	int size;

	size = strlen(str);
	if(off >= size)
		return 0;
	if(off+n > size)
		n = size-off;
	memmove(buf, str+off, n);
	return n;
}


static int
readetc(Ufile *file, void *buf, int len, vlong off)
{
	Etcfile *ef = (Etcfile *)file;

	return readstr(off, buf, len, ef->data);
}

static int
readdiretc(Ufile *, Udirent **pd)
{
	int q, n;

	n = 0;
	for(q = 0; q < nelem(etcdevtab); q++){
		if((*pd = newdirent(etcdevtab[q].name, etcdevtab[q].mode, mkino(ETCDEV, q))) == nil)
			break;
		pd = &((*pd)->next);
		n++;
	}
	return n;
}

static int
statetc(char *path, int link, Ustat *s)
{
	int q;

	if((q = path2q(path)) < 0)
		return -ENOENT;
	if(! link && (etcdevtab[q].mode & S_IFLNK) == S_IFLNK)
		return -SYMLINK;
	s->mode = etcdevtab[q].mode;
	if((s->mode & S_IFLNK) == S_IFLNK)
		s->size = strlen(etcdevtab[q].link);
	s->rdev = etcdevtab[q].rdev;
	s->ino = mkino(ETCDEV, q);
	s->dev = ETCDEV;
	s->atime = s->mtime = s->ctime = boottime/1000000000LL;
	return 0;
}

static vlong
sizeetc(Ufile *file)
{
	Etcfile *mf = (Etcfile *)file;

	if((etcdevtab[mf->q].mode & S_IFLNK) == S_IFLNK)
		return strlen(etcdevtab[mf->q].link);
	return strlen(mf->data);
}

static int
fstatetc(Ufile *file, Ustat *s)
{
	return statetc(file->path, 0, s);
};


static int
readlinketc(char *path, char *buf, int len)
{
	int n, q;

	if((q = path2q(path)) < 0)
		return -ENOENT;
	if((etcdevtab[q].mode & S_IFLNK) != S_IFLNK)
		return -EINVAL;
	n = strlen(etcdevtab[q].link);
	if(len > n)
		len = n;
	memcpy(buf, etcdevtab[q].link, len);
	return len;
}

static Udev etcdev =
{
	.name = "etc",
	.open = openetc,
	.read = readetc,
	.size = sizeetc,
	.close = closeetc,
	.stat = statetc,
	.fstat = fstatetc,
	.readdir = readdiretc,
	.readlink = readlinketc,
};

void
etcdevinit(void)
{
	devtab[ETCDEV] = &etcdev;

	fsmount(&etcdev, "/etc/passwd");
	fsmount(&etcdev, "/etc/group");
	fsmount(&etcdev, "/etc/resolv.conf");
}
