#pragma hjdicks on

enum {
	AF_UNIX			=1,
	AF_INET			=2,
	AF_INET6		=10,
	AF_NETLINK		=16,
};

enum {
	SOCK_STREAM		=1,
	SOCK_DGRAM		=2,
	SOCK_RAW		=3,

	/* can be or'ed into the above */
	SOCK_NONBLOCK 		=04000,
	SOCK_CLOEXEC 		=02000000,
};

enum {						/* socket ioctls */
	SIOCGIFNAME 		= 0x8910,	/* get interface name */
	SIOCGIFCONF 		= 0x8912,	/* get iface list */
	SIOCGIFFLAGS 		= 0x8913,	/* get flags */
	SIOCGIFADDR 		= 0x8915	/* get PA address */
};

enum {
						/* SIOCGIFFLAGS */
	IFF_UP 			= 0x01,		/* interface is up */
	IFF_LOOPBACK		= 0x08,		/* is a loopback I/F */
	IFF_NOTRAILERS		= 0x20,		/* avoid trailers */
	IFF_RUNNING 		= 0x40,		/* resources allocated */
	IFF_AUTOMEDIA 		= 0x4000	/* auto select media active */
};

	

enum {
	MSG_OOB		= 0x01,		/* Process out-of-band data */
	MSG_PEEK	= 0x02,		/* Peek at incoming messages */
};

enum {
	IFNAMSIZ	= 16,
	UNIX_PATH_MAX 	= 108,
};

typedef ushort	sa_family_t;
typedef ushort	in_port_t;
typedef uint	in_addr_t;


struct in_addr {
	in_addr_t s_addr;		/* address in network byte order */
};

struct sockaddr_in {
	sa_family_t sin_family;		/* address family: AF_INET */
	in_port_t sin_port;		/* port in network byte order */
	struct in_addr sin_addr;	/* internet address */
	char sin_zero[8];
};

struct sockaddr_nl {
	sa_family_t sin_family;		/* address family: AF_INET */
	ushort nl_pad;
	uint nl_pid;
	uint nl_groups;
};

struct sockaddr_un {
	sa_family_t sun_family;		/* AF_UNIX */
	char sun_path[UNIX_PATH_MAX];	/* pathname */
};

struct sockaddr {
	sa_family_t sa_family;		/* address family, AF_xxx */
	char sa_data[14];		/* 14 bytes of protocol address */
};

struct ifmap {
	ulong mem_start;
	ulong mem_end;
	ushort base_addr;
	uchar irq;
	uchar dma;
	uchar port;
};

struct ifreq {
	char ifr_name[IFNAMSIZ];	/* Interface name */
	union {
		struct sockaddr ifr_addr;
		struct sockaddr ifr_dstaddr;
		struct sockaddr ifr_broadaddr;
		struct sockaddr ifr_netmask;
		struct sockaddr ifr_hwaddr;
		short ifr_flags;
		int ifr_ifindex;
		int ifr_metric;
		int ifr_mtu;
		struct ifmap ifr_map;
		char ifr_slave[IFNAMSIZ];
		char ifr_newname[IFNAMSIZ];
		char *ifr_data;
	};
};

struct ifconf {
	int ifc_len;	       		/* size of buffer */
	union {
		char *ifc_buf; 		/* buffer address */
		struct ifreq *ifc_req;	/* array of structures */
	};
};

struct msghdr {
	uchar	*name;
	int	namelen;
	struct linux_iovec *iov;
	int	iovlen;
	void	*accrights;
	int	accrightslen;
};

#pragma hjdicks off
