#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

static char *argv0;

static void
usage(void)
{
	fprintf(stderr, "usage: %s [-nnnn] file ...\n", argv0);
	exit(1);
}

static void
plan9cwd(char *cwd, int len)
{
	int fd, n;
	char *file;
	char root[1024];

	getcwd(cwd, len);

	file = "/proc/plan9/rootpath";
	if((fd = open(file, O_RDONLY)) < 0){
		fprintf(stderr, "%s: %s cannot open %s\n", argv0, file, strerror(errno));
		exit(1);
	}
	n = read(fd, root, sizeof(root)-1);
	root[n] = 0;
	close(fd);

	if(strncmp(cwd, root, n) == 0)
		strcpy(cwd, cwd+n);
	if(strncmp(cwd, "/n/9", 4) == 0)
		strcpy(cwd, cwd+4);
}

static int
fixpath(char *buf, int len, char *file, char *cwd)
{
	if(*file == '/')
		return snprintf(buf, len, "%s", file);
	return snprintf(buf, len, "%s/%s", cwd, file);
}

int
main(int argc, char *argv[])
{
	int i, fd, n, len, line;
	char *send, *send2, cwd[1024], path[1024], buf[1024];

	send = "/n/9/mnt/plumb/send";
	send2 = "/n/9/mnt/term/mnt/plumb/send";
	argv0 = argv[0];
	if(argc < 2)
		usage();

	line = -1;
	if(argv[1][0] == '-'){
		line = - atoi(argv[1]);
		argv++;
		argc--;
	}

	if((fd = open(send, O_WRONLY)) < 0)
		if((fd = open(send2, O_WRONLY)) < 0){
			fprintf(stderr, "%s: %s cannot open %s\n", argv0, send, strerror(errno));
			exit(1);
		}

	plan9cwd(cwd, sizeof(cwd));
			
	for(i = 1; i < argc; i++){
		len = fixpath(path, sizeof(path), argv[i], cwd)+1;
		if(line == -1)
			n = snprintf(buf, sizeof(buf), "B\nedit\n%s\ntext\n\n%d\n%s\n",
				cwd, len, path);
		else
			n = snprintf(buf, sizeof(buf), "B\nedit\n%s\ntext\naddr=%d\n%d\n%s\n",
				cwd, line, len, path);
		if(write(fd, buf, n) != n)
			fprintf(stderr, "%s: %s write failed %s\n", argv0, send, strerror(errno));
	}
	close(fd);
}
