/* mc.c - print lines in multiple collumn */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#define	WIDTH			80
#define	TAB			4
#define	WORD_ALLOC_QUANTA	1024
#define	ALLOC_QUANTA		4096

int linewidth=WIDTH;
int mintab=1;
int colonflag=0;
int tabflag=0;	/* -t flag turned off forever */
char *cbuf, *cbufp;
char **word;
int maxwidth=0;
int nalloc=ALLOC_QUANTA;
int nwalloc=WORD_ALLOC_QUANTA;
int nchars=0;
int nwords=0;
int tabwidth=0;

#define nil NULL

void getwidth(void), readbuf(FILE *), error(char *);
void scanwords(void), columnate(void), morechars(void);
int wordwidth(char*, int);
int nexttab(int);

int
main(int argc, char *argv[])
{
	int i;
	int lineset;
	FILE *fp;

	lineset = 0;
	while(argc > 1 && argv[1][0] == '-'){
		--argc; argv++;
		switch(argv[0][1]){
		case '\0':
			colonflag = 1;
			break;
		case 't':
			tabflag = 0;
			break;
		default:
			linewidth = atoi(&argv[0][1]);
			if(linewidth <= 1)
				linewidth = WIDTH;
			lineset = 1;
			break;
		}
	}
	if(lineset == 0){
		getwidth();
		if(linewidth <= 1){
			linewidth = WIDTH;
		}
	}

	cbuf = cbufp = malloc(ALLOC_QUANTA*(sizeof *cbuf));
	word = malloc(WORD_ALLOC_QUANTA*(sizeof *word));
	if(word == 0 || cbuf == 0)
		error("out of memory");
	if(argc == 1)
		readbuf(stdin);
	else{
		for(i = 1; i < argc; i++){
			if((fp = fopen(*++argv, "r")) == nil)
				fprintf(stderr, "mc: can't open %s (%s)\n", *argv, strerror(errno));
			else{
				readbuf(fp);
				fclose(fp);
			}
		}
	}
	columnate();
	exit(0);
}

void
error(char *s)
{
	fprintf(stderr, "mc: %s\n", s);
	exit(-1);
}

void
readbuf(FILE *fp)
{
	int lastwascolon = 0;
	long c;
	int linesiz = 0;

	do{
		if(nchars++ >= nalloc)
			morechars();
		*cbufp++ = c = fgetc(fp);
		linesiz++;
		if(c == '\t') {
			cbufp[-1] = ' ';
			while(linesiz%TAB != 0) {
				if(nchars++ >= nalloc)
					morechars();
				*cbufp++ = ' ';
				linesiz++;
			}
		}
		if(colonflag && c == ':')
			lastwascolon++;
		else if(lastwascolon){
			if(c == '\n'){
				--nchars; 	/* skip newline */
				*cbufp = '\0';
				while(nchars > 0 && cbuf[--nchars] != '\n')
					;
				if(nchars)
					nchars++;
				columnate();
				if (nchars)
					putchar('\n');
				printf("%s", cbuf+nchars);
				nchars = 0;
				cbufp = cbuf;
			}
			lastwascolon = 0;
		}
		if(c == '\n')
			linesiz = 0;
	}while(c >= 0);
}

void
scanwords(void)
{
	char *p, *q;
	int i, w;

	nwords=0;
	maxwidth=0;
	for(p = q = cbuf, i = 0; i < nchars; i++){
		if(*p++ == '\n'){
			if(nwords >= nwalloc){
				nwalloc += WORD_ALLOC_QUANTA;
				if((word = realloc(word, nwalloc*sizeof(*word)))==0)
					error("out of memory");
			}
			word[nwords++] = q;
			p[-1] = '\0';
			w = wordwidth(q, p-q-1);
			if(w > maxwidth)
				maxwidth = w;
			q = p;
		}
	}
}

void
columnate(void)
{
	int i, j;
	int words_per_line;
	int nlines;
	int col;
	int endcol;


	scanwords();
	if(nwords==0)
		return;
	maxwidth = nexttab(maxwidth+mintab-1);
	words_per_line = linewidth/maxwidth;
	if(words_per_line <= 0)
		words_per_line = 1;
	nlines=(nwords+words_per_line-1)/words_per_line;
	for(i = 0; i < nlines; i++){
		col = endcol = 0;
		for(j = i; j < nwords; j += nlines){
			endcol += maxwidth;
			printf("%s", word[j]);
			col += wordwidth(word[j], strlen(word[j]));
			if(j+nlines < nwords){
				if(tabflag) {
					while(col < endcol){
						putchar('\t');
						col = nexttab(col);
					}
				}else{
					while(col < endcol){
						putchar(' ');
						col++;
					}
				}
			}
		}
		putchar('\n');
	}
}

int
wordwidth(char *w, int nw)
{
	return nw;
}

int
nexttab(int col)
{
	if(tabwidth){
		col += tabwidth;
		col -= col%tabwidth;
		return col;
	}
	return col+1;
}

void
morechars(void)
{
	nalloc += ALLOC_QUANTA;
	if((cbuf = realloc(cbuf, nalloc*sizeof(*cbuf))) == 0)
		error("out of memory");
	cbufp = cbuf+nchars-1;
}

#ifdef unix
#include <termio.h>
void
getwidth(void)
{
	struct winsize ws;

	if(ioctl(1, TIOCGWINSZ, &ws) == 0)
		linewidth = ws.ws_col;
}
#else
void
getwidth(void)
{
}
#endif
