/* p.c - print file in pages, after Kernighan and Pike 1984 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

#define DEFAULT 22

char *argv0;
FILE *tty;

int
runcmd(char *s)	
{
	int status, pid, w;
	void *istat, *qstat;
	char *shell, *name;

	if((shell = getenv("SHELL")) == NULL)
		shell = "/bin/sh";
	if((name = strrchr(shell, '/')) == NULL)
		name = "/sh";
	name++;

	if((pid = fork()) == 0){
		close(0); dup(fileno(tty));
		close(1); dup(fileno(tty));
		close(2); dup(fileno(tty));
		close(fileno(tty));
		execlp(shell, name, "-c", s, NULL);
		exit(127);
	}
	istat = signal(SIGINT, SIG_IGN);
	qstat = signal(SIGQUIT, SIG_IGN);
	while(((w = wait(&status)) != pid) && w != -1)
		continue;
	if(w == -1)
		status = -1;
	signal(SIGINT, istat);
	signal(SIGQUIT, qstat);
	return status;
}
	
int
chat(void)
{
	char c, chatbuf[128];

	fflush(stdout);
	while((c = fgetc(tty)) == '!')
		if(fgets(chatbuf, sizeof(chatbuf), tty))
			runcmd(chatbuf);

	if(c == 'n')
		return 1;
	if(c == 'q')
		exit(0);
	return 0;
}

void
page(FILE * fp, int nlines, int not_last)
{
	int c;
	long line = 0;

	while((c = fgetc(fp)) != -1)
		if(c == '\n' && (++line % nlines) == 0){
			if(chat())
				break;
		}
		else
			putchar(c);

	if(not_last && line < nlines)
		chat();
}

int
main(int argc, char *argv[])
{
	FILE *fp = stdin;
	int rc = 0, nlines = DEFAULT, multi = 0;

	if((tty = fopen("/dev/console", "r+")) == NULL){
		fprintf(stderr, "/dev/console - cannot open\n");
		tty = stdin;
	}

	argv0 = argv[0];
	if(argc > 1 && *argv[1] == '-')
		if((nlines = -atoi((argc--, *++argv))) == 0){
			fprintf(stderr, "usage: %s [-<n>] [file ...]\n",
				argv0);
			return 1;
		}

	while(argv++, --argc)
		if((fp = fopen(*argv, "r")) == NULL){
			fprintf(stderr, "%s: %s cannot open - %s\n", argv0,
				*argv, strerror(errno));
			rc++;
		}
		else {
			setvbuf(fp, NULL, _IOFBF, 8192);
			page(fp, nlines, (argc > 1));
			fclose(fp);
		}

	if(fp == stdin)
		page(stdin, nlines, multi);
	if (tty != stdin)
		fclose(tty);

	return rc;
}
